/*==========================================================================;
 *
 *  File:          Mutex.cpp
 *  Version:       1.0
 *  Desc:		   Implements mutexes for multithreading
 *  Author:        Miha Grcar
 *  Created on:    Jul-2007
 *  Last modified: Sep-2007
 *  License:       Common Public License (CPL)
 * 
 ***************************************************************************/

#include "Mutex.h"
#include <assert.h>

void Mutex::create(const char *name, int timeout_ms, bool wait)
{
    HANDLE h = OpenMutex(MUTEX_ALL_ACCESS, FALSE, name);
    if (h == 0) 
    {
        h = (HANDLE)CreateMutex(0, FALSE, name);
        assert(h != 0);
    }
    m_handle = h;
    m_timeout_ms = timeout_ms;
    m_ref_count = 0;
    if (wait) { Mutex::wait(); }
}

Mutex::~Mutex()
{
    if (m_ref_count > 0) { release(); }
    assert(m_ref_count == 0 && m_handle != 0);	
    CloseHandle(m_handle);
}

void Mutex::wait()
{
    DWORD ret_val = WaitForSingleObject(m_handle, m_timeout_ms);
    assert(ret_val != WAIT_ABANDONED && ret_val != WAIT_TIMEOUT);
    m_ref_count++;
}

void Mutex::release()
{
    m_ref_count--;
    ReleaseMutex(m_handle);
}