/*==========================================================================;
 *
 *  File:          Mutex.h
 *  Version:       1.0
 *  Desc:		   Implements mutexes for multithreading
 *  Author:        Miha Grcar
 *  Created on:    Jul-2007
 *  Last modified: Sep-2007
 *  License:       Common Public License (CPL)
 *  Note:
 *		One way to use these mutexes is to simply start a new scope and define
 *      a new variable of type Mutex in the first line. This variable *should*
 *      be automatically destroyed when going out of scope thus the mutex should 
 *		be released. However, you need to name the variable, e.g. Mutex foo("bar"),
 *      otherwise it will be destroyed right after being created. It is safer to
 *		call wait() and release() explicitly as this behavior might be compiler-
 *      specific.
 * 
 ***************************************************************************/

#ifndef MUTEX_H
#define MUTEX_H

#include <windows.h>

class Mutex
{
private:
    int m_timeout_ms;
    HANDLE m_handle;
    int m_ref_count;
public:
    Mutex(const char *name) 
    { create(name, 5000, true); }
    Mutex(const char *name, int timeout_ms) 
    { create(name, timeout_ms, true); }
    Mutex(const char *name, int timeout_ms, bool wait) 
    { create(name, timeout_ms, wait); }
    ~Mutex();
    void wait();
    void release();
private:
    void create(const char *name, int timeout_ms, bool wait);
};

#endif